@@ -3,7 +3,7 @@
 #
 # Author: Slaven Rezic
 #
-# Copyright (C) 2006,2008,2009,2012,2014 Slaven Rezic. All rights reserved.
+# Copyright (C) 2006,2008,2009,2012,2014,2015 Slaven Rezic. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 #
@@ -21,12 +21,12 @@ use 5.006; # qr, autovivified filehandles
 use strict;
 use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
 
-$VERSION = '0.09';
+$VERSION = '0.10';
 
 require Exporter;
 @ISA = qw(Exporter);
 @EXPORT    = qw(xterm_conf);
-@EXPORT_OK = qw(xterm_conf_string);
+@EXPORT_OK = qw(xterm_conf_string terminal_is_supported);
 
 use Getopt::Long 2.24; # OO interface
 
@@ -73,7 +73,6 @@ sub xterm_conf_string {
     local @ARGV = @_;
 
     %o = ();
-    $need_reset_terminal++;
 
     my $p = Getopt::Long::Parser->new;
     $p->configure('no_ignore_case');
@@ -235,11 +234,24 @@ sub xterm_conf_string {
 }
 
 sub xterm_conf {
-    return if !$ENV{TERM};
-    return if $ENV{TERM} !~ m{^(xterm|rxvt)};
+    # always call xterm_conf_string(), so option validation is done
     my $rv = xterm_conf_string(@_);
-    local $| = 1;
-    print $rv;
+    if (terminal_is_supported()) {
+	local $| = 1;
+	print $rv;
+    }
+}
+
+sub terminal_is_supported {
+    my($term) = @_;
+    $term = $ENV{TERM} if !defined $term;
+    if (!$ENV{TERM}) {
+	0;
+    } elsif ($ENV{TERM} !~ m{^(xterm|rxvt)}) {
+	0;
+    } else {
+	1;
+    }
 }
 
 sub _report ($$) {
@@ -248,30 +260,35 @@ sub _report ($$) {
     require Term::ReadKey;
     Term::ReadKey::ReadMode(5);
 
-    require IO::Select;
-
-    my $debug = $o{debugreport};
-
-    open my $TTY, "+< /dev/tty" or die "Cannot open terminal /dev/tty: $!";
-    syswrite $TTY, $cmd;
-
-    my $sel = IO::Select->new;
-    $sel->add($TTY);
-
-    my $res = "";
     my @args;
-    my $err;
-    while() {
-	my(@ready) = $sel->can_read(5);
-	if (!@ready) {
-	    $err = "Cannot report, maybe allowWindowOps is set to false?";
-	    last;
+
+    eval {
+	require IO::Select;
+
+	my $debug = $o{debugreport};
+
+	open my $TTY, "+< /dev/tty" or die "Cannot open terminal /dev/tty: $!";
+	syswrite $TTY, $cmd;
+
+	my $sel = IO::Select->new;
+	$sel->add($TTY);
+
+	my $res = "";
+	while() {
+	    my(@ready) = $sel->can_read(5);
+	    if (!@ready) {
+		die "Cannot report, maybe allowWindowOps is set to false?";
+		last;
+	    }
+	    sysread $TTY, my $ch, 1 or die "Cannot sysread: $!";
+	    print STDERR ord($ch)." " if $debug;
+	    $res .= $ch;
+	    last if (@args = $res =~ $rx);
 	}
-	sysread $TTY, my $ch, 1 or die "Cannot sysread: $!";
-	print STDERR ord($ch)." " if $debug;
-	$res .= $ch;
-	last if (@args = $res =~ $rx);
-    }
+
+	1;
+    };
+    my $err = $@;
 
     Term::ReadKey::ReadMode(0);
 
@@ -311,11 +328,6 @@ usage: $0 [-n|iconname string] [-T|title string] [-cr|textcursor color]
 EOF
 }
 
-END {
-    Term::ReadKey::ReadMode(0)
-	    if $need_reset_terminal && defined &Term::ReadKey::ReadMode;
-}
-
 return 1 if caller;
 
 xterm_conf(@ARGV);
@@ -615,6 +627,13 @@ performed here.
 
 xterm_conf_string may be exported.
 
+=head2 terminal_is_supported(I<term>)
+
+Return a true value if the given I<term>, or if missing, the current
+terminal as given by C<$ENV{TERM}>, is supported.
+
+This function may be exported.
+
 =head1 AUTHOR
 
 Slaven ReziE<0x107>
